/* usbreset -- send a USB port reset to a USB device */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>

#include <linux/usbdevice_fs.h>


int main(int argc, char **argv)
{
    const char *filename;
    int fd;
    int rc;

    if (argc != 2) {
        fprintf(stderr, "Usage: %s [device-filename]\n", argv[0]);
        fprintf(stderr, "\n");
        fprintf(stderr, "Example: sudo %s /dev/bus/usb/[BUS]/[DEVICE]\n", argv[0]);
        fprintf(stderr, "Use lsusb to find the bus and device.\n");
        return EXIT_FAILURE;
    }
    filename = argv[1];

    fd = open(filename, O_WRONLY);
    if (fd < 0) {
        perror("Error opening output file");
        return EXIT_FAILURE;
    }

    printf("Resetting USB device %s\n", filename);
    rc = ioctl(fd, USBDEVFS_RESET, 0);
    if (rc < 0) {
        perror("Error in ioctl");
        return EXIT_FAILURE;
    }
    printf("Reset successful\n");

    close(fd);
    return EXIT_SUCCESS;
}
